package F5::Policy::Convert::DosProfile;

use strict;
use warnings;

use F5::ASMConfig::GuaranteedClient;
use F5::Logger;
use F5::DbUtils;

our $MITIGATION_LEVEL_MAP = {
    'none'=> F5::ASMConfig::MitigationLevel::MITIGATION_LEVEL_NONE(),
    'conservative'=> F5::ASMConfig::MitigationLevel::MITIGATION_LEVEL_CONSERVATIVE(),
    'standard'=> F5::ASMConfig::MitigationLevel::MITIGATION_LEVEL_STANDARD(),
    'aggressive'=> F5::ASMConfig::MitigationLevel::MITIGATION_LEVEL_STANDARD(),
};

#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;

    my $self = {};
    bless $self, $class;
    $self->init(%params);

    return $self;
}

#----------------------------------------------------------------------

sub init {
    my ($self, %params) = @_;

    $self->{logger} = $params{logger} || F5::Logger->new('/ts/log/policy_convert.log');
    $self->{client} = $params{client};
    $self->{dbh} =  $params{dbh} || F5::DbUtils::get_dbh();
    if (! $self->{client}) {
        $self->{client} = F5::ASMConfig::GuaranteedClient::get_client((username => 'import_export_policy'));
    } 
}

#----------------------------------------------------------------------

sub import_dos_profile {
    my($self, %params) = @_;

    my $policy = $params{policy};
    my $dos_profile_json = $params{dos_profile};
    my $call_flags = $params{call_flags};

    for my $obj (values %$dos_profile_json) {
        for my $dos_profile (values %{$obj->{'application'}}) {
            if (defined $dos_profile->{'stress-based'} && defined $dos_profile->{'stress-based'}->{'mode'}) {
                my $flg_enabled = (grep { $_ eq $dos_profile->{'stress-based'}->{'mode'}} ('blocking', 'transparent')) ? 1 : 0;

                $self->{client}->update_dos_protection($self->{client}->{session},
                                                       $call_flags,
                                                       $policy,
                                                       F5::ASMConfig::DosProtection->new({flg_enabled => $flg_enabled}),
                                                      );

                if ($flg_enabled && defined $dos_profile->{'stress-based'}->{'behavioral'}) {
                    my $behavioral_profile = $dos_profile->{'stress-based'}->{'behavioral'};

                    my %behavioral_dos_params = (
                        mitigation_level => $MITIGATION_LEVEL_MAP->{$behavioral_profile->{'mitigation-mode'} || 'standard'},
                        flg_http_signatures => $self->_is_enabled($behavioral_profile->{'signatures'}),
                        flg_tls_signatures => $self->_is_enabled($behavioral_profile->{'tls-signatures'}),
                        flg_bad_actor_detection => $self->_is_enabled($behavioral_profile->{'dos-detection'}),
                        flg_bad_actor_detection_tls_indexing => $self->_is_enabled($behavioral_profile->{'tls-fp'}),
                    );

                    my $behavioral_dos = F5::ASMConfig::BehavioralDos->new(%behavioral_dos_params);

                    $self->{client}->update_behavioral_dos($self->{client}->{session},
                                                           $call_flags,
                                                           $policy,
                                                           $behavioral_dos,
                                                          );
                }
            }
        }
    }
    return;
}

#----------------------------------------------------------------------

sub _is_enabled {
    my($self, $val) = @_;

    return ($val && $val eq 'enabled') || 0;
}

#----------------------------------------------------------------------

1;