package F5::REST::WAF::ConvertPolicy;

use strict;
use warnings;

use File::Slurp qw(read_file);

use F5::Deploy;
use F5::Policy::Convert;

use base ('F5::REST::WAF');

#----------------------------------------------------------------------

sub process {
    my ($self, %params) = @_;

    my $c = $self->{connection};

    my $output_filename = $c->param('output-filename');
    my $source_policy_file = $c->req->upload('source-policy');
    my $bot_profile = $c->param('bot-profile');
    my $dos_profile = $c->param('dos-profile');
    if (! $source_policy_file && ! $bot_profile && ! $dos_profile) {
        $c->render(json => { message => "Source policy file not found" }, status => 404);
        return;
    }

    my $tmp_path = F5::Deploy::get_path_by_type('tmp');
    my $timestamp = time();

    my $import_filename;
    if ($source_policy_file) {
        $import_filename = $source_policy_file->filename;
        $import_filename = "$tmp_path/waf_upload_${timestamp}_". $import_filename;
        $source_policy_file->asset->move_to($import_filename);
    }

    my $bot_profile_data;
    if ($bot_profile) {
        $bot_profile = F5::YAML::Schema::Utils::parse_json(content => $bot_profile);
        $bot_profile_data = $bot_profile->{data};
    }

    my $dos_profile_data;
    if ($dos_profile) {
        $dos_profile = F5::YAML::Schema::Utils::parse_json(content => $dos_profile);
        $dos_profile_data = $dos_profile->{data};
    }

    my $export_filename = "$tmp_path/waf_download_${timestamp}_". $output_filename;

    my %convert_policy_params = (
        export_file => $export_filename,
        import_file => $import_filename,
        bot_profile => $bot_profile_data,
        dos_profile => $dos_profile_data,
        format => 'json',
        full_export => 1,
    );

    my $result;
    eval {
        my $converter = F5::Policy::Convert->new(logger => $self->{logger},);
        $result = $converter->convert_policy(%convert_policy_params);
    };

    if ($@) {
        $c->render(json => { message => $@ }, status => 404);
    }
    else {
        if ($result->{completed_successfully}) {
            my $content = read_file($result->{filename});
            $c->res->headers->content_type("application/x-download");
            $c->res->headers->content_disposition("attachment; filename=\"$output_filename\"");
            $c->render(data => $content);
        }
        else {
            $c->render(json => $result, status => 404);
        }
    }

    unlink $import_filename if $import_filename && -e $import_filename;
    unlink $export_filename if -e $export_filename;
}

#----------------------------------------------------------------------

1;

